static const char QTGUI_CONF[] =
"[appearance]\n"
"Skin = basic-ncb\n"
"Icons = ami\n"
"Emoticons = ccmathteam.com\n"
"ExtendedIcons = basic\n"
"Font = default\n"
"GridLines = 0\n"
"ShowHeader = 1\n"
"ShowOfflineUsers = 1\n"
"ShowDividers = 1\n"
"UseDock = 0\n"
"FontStyles = 1\n"
"NumColumns = 1\n"
"Column1.Title = Alias\n"
"Column1.Format = %a\n"
"Column1.Width = 90\n"
"Column1.Align = 0\n"
"\n"
"[startup]\n"
"Logon = 0\n"
"AutoAway = 5\n"
"AutoNA = 10\n"
"\n"
"[functions]\n"
"AutoClose = 1\n"
"\n"
"[files]\n"
"[floaties]\n"
"[locale]\n";







