static const char LICQ_CONF[] =
"\n"
"[licq]\n"
"Version = 1028\n"
"\n"
"[plugins]\n"
"NumPlugins = 0\n"
"NumProtoPlugins = 0\n"
"\n"
"[network]\n"
"AllowNewUsers = 1\n"
"NumOfICQServers = 1\n"
"ICQServer = login.icq.com\n"
"ICQServerPort = 5190\n"
"TCPPortsLow = 0\n"
"TCPPortsHigh = 0\n"
"TCPEnabled = 1\n"
"MaxUsersPerPacket = 100\n"
"IgnoreTypes = 0\n"
"ForegroundColor = 0\n"
"BackgroundColor = 16777215\n"
"Errors = log.errors\n"
"ErrorTypes = 6\n"
"Rejects = log.rejects\n"
"Translation = none\n"
"Terminal = xterm -T Licq -e \n"
"ProxyEnabled = 0\n"
"ProxyServerType = 1\n"
"ProxyServer = \n"
"ProxyServerPort = 0\n"
"ProxyAuthEnabled = 0\n"
"ProxyLogin = \n"
"ProxyPassword = \n"
"UseSS = 1\n"
"\n"
"[onevent]\n"
"Enable = 1\n"
"Command = play\n"
"\n"
"[groups]\n"
"NumOfGroups = 3\n"
"DefaultGroup = 0\n"
"NewUserGroup = 1\n"
"Group1.name = New Users\n"
"Group2.name = Friends\n"
"Group3.name = Family\n"
"Group1.id = 0\n"
"Group2.id = 0\n"
"Group3.id = 0\n"
"\n"
"[owners]\n"
"NumOfOwners = 0\n"
"Owner1.Id = 0\n"
"Owner1.PPID = Licq\n";
