#ifndef AUTORESPONSE_CONF_H
#define AUTORESPONSE_CONF_H

static const char SAR_CONF[] = {
"\n"
"[Away]\n"
"NumSAR = 7\n"
"SAR1.Name = Away\n"
"SAR2.Name = Lunch\n"
"SAR3.Name = Be right back\n"
"SAR4.Name = Dog Walk\n"
"SAR5.Name = Smoke\n"
"SAR6.Name = Coffee\n"
"SAR7.Name = Air\n"
"SAR1.Text = I am currently away from ICQ.\\nPlease leave your message and I will get back to you as soon as I return!\n"
"SAR2.Text = I'm out to lunch.\\nI will return shortly.\n"
"SAR3.Text = Don't go anywhere!\\nI'll be back in a jiffy.\n"
"SAR4.Text = I'm out with the dog.\\nBe back when he's finished\n"
"SAR5.Text = Went out for a smoke.\n"
"SAR6.Text = On my coffee break.\n"
"SAR7.Text = Went to get some fresh air.\n"
"\n"
"[NA]\n"
"NumSAR = 7\n"
"SAR1.Name = Out for the day\n"
"SAR2.Name = Not here\n"
"SAR3.Name = Closed\n"
"SAR4.Name = Fishing\n"
"SAR5.Name = Sleeping\n"
"SAR6.Name = Kids\n"
"SAR7.Name = Gone\n"
"SAR1.Text = I am out'a here.\\nSee you tomorrow!\n"
"SAR2.Text = Give it up! I 'm not in!\n"
"SAR3.Text = I'm closed for the weekend/holidays.\n"
"SAR4.Text = Gone fishin'!\n"
"SAR5.Text = I'm sleeping. Don't wake me.\n"
"SAR6.Text = Went home.\\nHad to feed the kids.\n"
"SAR7.Text = Gone for good.\n"
"\n"
"[Occupied]\n"
"NumSAR = 7\n"
"SAR1.Name = Busy\n"
"SAR2.Name = Meeting\n"
"SAR3.Name = Concentration\n"
"SAR4.Name = On the Phone\n"
"SAR5.Name = Can't chat\n"
"SAR6.Name = Working\n"
"SAR7.Name = Conversing\n"
"SAR1.Text = Please do not disturb me now.\\nDisturb me later.\\nOnly urgent messages, please!\n"
"SAR2.Text = I'm currently in a meeting. I can't be disturbed.\\nOnly urgent messages, please!\n"
"SAR3.Text = Don't disturb my concentration!\\nOnly urgent messages, please!\n"
"SAR4.Text = I'm on the phone with a very important client. Don't disturb me!\\nOnly urgent messages please!\n"
"SAR5.Text = I can't chat with you now. I'm busy.\\nOnly urgent messages, please!\n"
"SAR6.Text = Can't you see I'm working?\\nOnly urgent messages, please!\n"
"SAR7.Text = I am conversing with my colleagues.\\nOnly urgent messages, please!\n"
"\n"
"[DND]\n"
"NumSAR = 7\n"
"SAR1.Name = Do Not Disturb\n"
"SAR2.Name = Meeting\n"
"SAR3.Name = Concentration\n"
"SAR4.Name = On the Phone\n"
"SAR5.Name = Can't chat\n"
"SAR6.Name = Working\n"
"SAR7.Name = Fiddling\n"
"SAR1.Text = Please do not disturb me now. Disturb me later.\n"
"SAR2.Text = I'm currently in a meeting.\\nI can't be disturbed.\n"
"SAR3.Text = Don't disturb my concentration!\n"
"SAR4.Text = I'm on the phone with a very important client.\\nDon't disturb me!\n"
"SAR5.Text = I can't chat with you now. I'm busy.\n"
"SAR6.Text = Can't you see I'm working?\n"
"SAR7.Text = Fiddling around some things and I'm quite busy with that.\\nSo please disturb me later!\n"
"\n"
"[FFC]\n"
"NumSAR = 5\n"
"SAR1.Name = Chat\n"
"SAR2.Name = Come in\n"
"SAR3.Name = Fun\n"
"SAR4.Name = Don't Wait\n"
"SAR5.Name = Hear\n"
"SAR1.Text = We'd love to hear what you have to say. Join our chat.\n"
"SAR2.Text = Come in and join my chat room!\n"
"SAR3.Text = Don't miss the fun! Join our chat!\n"
"SAR4.Text = What are you waiting for? Come in!\n"
"SAR5.Text = We'd love to hear\\nwhat you have to say.\\nJoin our chat!\n"
"\n"
};

#endif
